/*=========================================================================

  Program:   Insight Segmentation & Registration Toolkit
  Module:    $RCSfile: qtITK.cxx,v $
  Language:  C++
  Date:      $Date: 2002/09/16 02:43:06 $
  Version:   $Revision: 1.9 $

  Copyright (c) 2002 Insight Consortium. All rights reserved.
  See ITKCopyright.txt or http://www.itk.org/HTML/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/



#include <qapplication.h>
#include <qpushbutton.h>
#include <qfiledialog.h>
#include <qslider.h>

#include "itkImage.h"
#include "itkImageFileReader.h"
#include "itkCurvatureFlowImageFilter.h"


#include "itkQtProgressBar.h"

// files generated by moc - QT4
#include "itkQtAdaptor.moc"
#include "itkQtLightIndicator.moc"


int main(int argc, char **argv)
{

  std::cout<< "Line1" << std::endl;

  typedef itk::Image< float, 2 > ImageType;

  typedef itk::CurvatureFlowImageFilter< 
                    ImageType,
                    //ImageType,
                    ImageType >  FilterType;

  typedef itk::ImageFileReader< ImageType > ReaderType;

  ReaderType::Pointer reader = ReaderType::New();
  
  FilterType::Pointer filter = FilterType::New();

  std::cout << "Now building ITK-Curvature Flow: " << std::endl << std::endl;
  filter->SetTimeStep(5.0);
  filter->SetNumberOfIterations(100);
  filter->SetInput( reader->GetOutput() );
  //cf->Update();

  //filter->SetInput2( reader->GetOutput() );

  // Create Qt Application to let Qt get its 
  // parameters from the command line
  QApplication app( argc, argv );

  QWidget qb;
  qb.resize(620,200);

  const int buttonHeight =  30;
  const int buttonWidth  = 100;
  const int buttonSpace  = 100;

  int horizontalPosition =  60;

  QPushButton  bb( "Start", &qb );
  bb.setGeometry( horizontalPosition, 20, buttonWidth, buttonHeight );

  horizontalPosition += buttonWidth + buttonSpace;

  QtLightIndicator  cc( &qb, "State" );
  cc.setGeometry( horizontalPosition, 20, buttonWidth, buttonHeight );
  cc.Modified();

  horizontalPosition += buttonWidth + buttonSpace;

  QPushButton  qt( "Quit", &qb );
  qt.setGeometry( horizontalPosition, 20, buttonWidth, buttonHeight );

  QtProgressBar qs( &qb, "Progress");
  qs.setGeometry( 10, 60, 600, 30 );
  
  QSlider sl(Qt::Horizontal, &qb);
  sl.setRange(1, 100);
  sl.setValue(10);
  sl.setGeometry(100, 110, 400, 60);

  // Connect the progress bar to the ITK processObject
  qs.Observe( filter.GetPointer() );
  qs.Observe( reader.GetPointer() );


  typedef QtSlotAdaptor<FilterType> SlotAdaptorType;
  SlotAdaptorType slotAdaptor1;
  // Connect the adaptor to a method of the ITK filter
  slotAdaptor1.SetCallbackFunction( filter, & FilterType::Update );
  // Connect the adaptor's Slot to the Qt Widget Signal
  QObject::connect( &bb, SIGNAL(clicked()), &slotAdaptor1, SLOT(Slot()) );

  typedef QtSlotAdaptor<FilterType> SlotAdaptorType;
  SlotAdaptorType slotAdaptor2;
  // Connect the adaptor to a method of the ITK filter
  slotAdaptor2.SetCallbackFunction( filter, & FilterType::SetNumberOfIterations );
  // Connect the adaptor's Slot to the Qt Widget Signal
  QObject::connect( &sl, SIGNAL(sliderMoved(int)), &slotAdaptor2, SLOT(Slot()) );


  // One signal adaptor for observing the StartEvent()
  typedef QtSignalAdaptor SignalAdaptorType;
  SignalAdaptorType signalAdaptor1;
  // Connect the adaptor as an observer of a Filter's event
  filter->AddObserver( itk::StartEvent(),  signalAdaptor1.GetCommand() );
  // Connect the adaptor's Signal to the Qt Widget Slot
  QObject::connect( &signalAdaptor1, SIGNAL(Signal()), &cc, SLOT(Start()) );


  // One signal adaptor for observing the ModifiedEvent()
  typedef QtSignalAdaptor SignalAdaptorType;
  SignalAdaptorType signalAdaptor2;
  // Connect the adaptor as an observer of a Filter's event
  filter->AddObserver( itk::ModifiedEvent(),  signalAdaptor2.GetCommand() );
  // Connect the adaptor's Signal to the Qt Widget Slot
  QObject::connect( &signalAdaptor2, SIGNAL(Signal()), &cc, SLOT(Modified()) );


  // One signal adaptor for observing the EndEvent()
  typedef QtSignalAdaptor SignalAdaptorType;
  SignalAdaptorType signalAdaptor3;
  // Connect the adaptor as an observer of a Filter's event
  filter->AddObserver( itk::EndEvent(),  signalAdaptor3.GetCommand() );
  // Connect the adaptor's Signal to the Qt Widget Slot
  QObject::connect( &signalAdaptor3, SIGNAL(Signal()), &cc, SLOT(End()) );


  // Connect the Quit button signal to the quit slot of the application
  QObject::connect( &qt, SIGNAL(clicked()), &app, SLOT(quit()) );

  //qb.setFocus();
  app.setActiveWindow ( &qb );
  qb.show();

  //QString s = QFileDialog::getOpenFileName( 0, "open file dialog","Chose an image filename" );

  QString s = QFileDialog::getOpenFileName( 0, "Chose an image filename", ".", "Images (*.png *.mha *.jpg)");

  reader->SetFileName( s.toLatin1() );

  return app.exec();

}



